==> healthcheck-0.1.0-20161011.173434-3.jar: healthcheck.clj <==
(ns healthcheck
  "Health checks for Ring, following the reloaded pattern.
   Provides a component for configuration and a configurable endpoint.
  "
  (:require
    [com.stuartsierra.component :as component]
    [compojure.core :refer [GET routes]]
    [ring.util.http-response :refer :all]))

(defrecord Healthcheck [config]
  component/Lifecycle
  (start [this] this)
  (stop [this]this))

(defn healthcheck [config]
  (->Healthcheck config))


(def default-healthchecks
  {::basic (fn [] "OK")})

(defn healthcheck-result [check]
  (try
    (let [check-result (check)]
      (if check-result
        [::success check-result]
        [::failure check-result]))
    (catch Exception e
      [::failure e])))

(defn success?
  "Returns true if ALL checks succeed."
  [healthcheck-results]
  (empty? (filter #(not= ::success (first (second %))) healthcheck-results)))

(defn run [healthcheck-map]
  (into {} (map (fn [[name check]] [name (healthcheck-result check)])
                healthcheck-map)))

(defn endpoint
  "Creates a healthcheck route for Ring. Configuration may be passed to config
  in the :healthcheck key.

  Configuration options:

    - healthchecks:     A map of keyword check name and check-fn pairs.
    - path              The path of the healthcheck route. Default \"_health\"
  
  Example:
      (endpoint {:healthcheck {:healthchecks default-healthchecks
                               :path         \"_health\"}})"
  [{{healthchecks :healthchecks
    path :path
    :or  {healthchecks default-healthchecks
          path "_health"}}
    :healthcheck}]
  (routes
    (GET path []
         :name    :healthcheck
         :summary "Returns healthcheck information."
         (let [result (run healthchecks)]
           (if (success? result)
             (ok result)
             (internal-server-error! result))))))

==> healthcheck-0.1.0-20161011.183457-4.jar: healthcheck.clj <==
(ns healthcheck
  "Health checks for Ring, following the reloaded pattern.
   Provides a component for configuration and a configurable endpoint.
  "
  (:require
    [com.stuartsierra.component :as component]
    [compojure.api.sweet :refer [GET routes]]
    [ring.util.http-response :refer :all]))

(defrecord Healthcheck [config]
  component/Lifecycle
  (start [this] this)
  (stop [this]this))

(defn healthcheck [config]
  (->Healthcheck config))


(def default-healthchecks
  {::basic (fn [] "OK")})

(defn healthcheck-result [check]
  (try
    (let [check-result (check)]
      (if check-result
        [::success check-result]
        [::failure check-result]))
    (catch Exception e
      [::failure e])))

(defn success?
  "Returns true if ALL checks succeed."
  [healthcheck-results]
  (empty? (filter #(not= ::success (first (second %))) healthcheck-results)))

(defn run [healthcheck-map]
  (into {} (map (fn [[name check]] [name (healthcheck-result check)])
                healthcheck-map)))

(defn endpoint
  "Creates a healthcheck route for Ring. Configuration may be passed to config
  in the :healthcheck key.

  Configuration options:

    - healthchecks:     A map of keyword check name and check-fn pairs.
    - path              The path of the healthcheck route. Default \"_health\"
  
  Example:
      (endpoint {:healthcheck {:healthchecks default-healthchecks
                               :path         \"_health\"}})"
  [{{healthchecks :healthchecks
    path :path
    :or  {healthchecks default-healthchecks
          path "_health"}}
    :healthcheck}]
  (routes
    (GET path []
         :name    :healthcheck
         :summary "Returns healthcheck information."
         (let [result (run healthchecks)]
           (if (success? result)
             (ok result)
             (internal-server-error! result))))))
